/*
         Object Oriented Programming using C++,
         Solution Manual and Applied Examples.
    	    written by Dr. Farschad Torabi
         K. N. Toosi University of Technology  
    	           September, 2013
    
                    Chapter 24
Code written by: Masoud Vahidi & Amirhossein Borji
*/

#include <iostream>
#include < fstream>
#include <string>
using namespace std;

//--Parent class------------------------------------------------------------------------//
class ACS
{
public:
	
	double *T ,*w ,*Fi ,*Pg ,*mda ,*h;
	double P,Q;

	void Calw (int i)
	{
		w[i] = (0.622*Fi[i]*Pg[i])/(P - Pg[i]);
	}

	void get_AirInfo(int n)
	{
		int i,j;
		double *s,*t;
		
		ifstream fin("air table.txt");
		
		cout<<"Please enter Temperature("<<n+1<<") of the room (C): ";
		cin>>T[n];
		T[n] = T[n] + 273;
		
		fin>>j;
		
		t = new double[j];
		s = new double[j];
		
		for(i=0;i<j;i++)
		{
			fin>>t[i];
			fin>>s[i];
			if(t[i]==T[n])
			{
				h[n] = s[i];
				break;
			}
		}
		
		for(i=0;i<j-1;i++)
		{	
			if(t[i]<T[n]&&T[n]<t[i+1])
			{
				h[n] = ((T[n]-t[i])/(t[i+1]-t[i]))*(s[i+1]-s[i]) + s[i];
				break;
			}
		}
		fin.close();
		delete [] t;
		delete [] s;
	}
	
	void get_VapInfo(int n)
	{
		int i,j;
		double *s,*t;
		
		ifstream fin("vapor table.txt");
		
		T[n] = T[n] - 273;
		
		fin>>j;
		
		t = new double[j];
		s = new double[j];
		
		for(i=0;i<j;i++)
		{
			fin>>t[i];
			fin>>s[i];
		
			if(t[i]==T[n])
			{
				Pg[n] = s[i]; 
				break;
			}
		}
		
		for(i=0;i<j;i++)
		{
			if(t[i]<T[n]&&T[n]<t[i+1])
			{
				Pg[n] = ((T[n]-t[i])/(t[i+1]-t[i]))*(s[i+1]-s[i]) + s[i];
				break;
			}
		}
		fin.close();
		delete [] t;
		delete [] s;
	}

	void get_P()
	{
		cout<<"Please enter Atmospheric Pressure of the room (KPa): ";
		cin>>P;
	}

	void set_w2 ()
	{
		w[1] = w[0];
	}

	void get_Fi1()
	{
		cout<<"Please enter first Relative humidity of the room (%): ";
		cin>>Fi[0];
	}
	
	void CalFi (int i)
	{
		Fi[i] = (w[i]*(P-Pg[i]))/(0.622*Pg[i]); 
	}

	void get_md1()
	{
		cout<<"Please enter internal Mass flow rate of dry air (Kg/s): ";
		cin>>mda[0];
	}

	void set_mda2()
	{
		mda[1] = mda[0];
	}
	
	void newing(int n)
	{
		T = new double[n];
		w = new double[n];
		Fi = new double[n];
		mda = new double[n];
		h = new double[n];
		Pg = new double[n];
	}
	
	void deleting()
	{
		delete [] T;
		delete [] w;
		delete [] Fi;
		delete [] mda;
		delete [] h;
		delete [] Pg;
	}
	
	virtual void CalQ() = 0;
	virtual void Print_info() = 0;
};

//--1st Child class---------------------------------------------------------------------//
class simple_HandC : public ACS
{
public:
	
	simple_HandC (int n)
	{
		ACS::newing(n);
	}
	
	~simple_HandC()
	{
		ACS::deleting();
	}
	
	virtual void CalQ()
	{
		Q = mda[0]*(h[1] - h[0]);
	}
	
	virtual void Print_info()
	{
		cout<<endl<<"Temperature 1 = "<<T[0]<<" (C)"<<endl<<"Temperature 2 = "<<T[1]<<" (C)"<<endl<<endl;
		cout<<"Relative humidity 1 = "<<Fi[0]<<endl<<"Relative humidity 2 = "<<Fi[1]<<endl<<endl;
		cout<<"Mass flow rate of dry air 1 = "<<mda[0]<<" (Kg/s)"<<"Mass flow rate of dry air 2 = "<<mda[1]<<" (Kg/s)"<<endl<<endl;
		cout<<"......................................................................."<<endl;
		cout<<"Saturated pressure 1 = "<<Pg[0]<<" (KPa)"<<endl<<"Saturated pressure 2 = "<<Pg[1]<<" (KPa)"<<endl<<endl;
		cout<<"Absolute humidity 1 = "<<w[0]<<" (Kg/Kg dry air)"<<endl<<"Absolute humidity 2 = "<<w[1]<<" (Kg/Kg dry air)"<<endl<<endl;
		cout<<"Enthalpy 1 = "<<h[0]<<" (KJ/Kg)"<<endl<<"Enthalpy 2 = "<<h[1]<<" (KJ/Kg)"<<endl<<endl;
		cout<<"Rate of heat transfer= "<<Q<<" (KJ/s)"<<endl<<endl;
	}
};

//--2nd Child class---------------------------------------------------------------------//
class HumandDeHum_HandC : public ACS
{
public:
	
	double mdw,hw,Tw;

	void get_Fi3()
	{
		cout<<"Please enter the Relative humdity that you want (%):" ;
		cin>>Fi[2];
	}

	void get_Tw()
	{
		cout<<"Please enter water Temperature of the humidifier or dehumidifier (C): ";
		cin>>Tw;
	}

	void get_WaterInfo()
	{
		int i,j;
		double *a,*c;
		
		ifstream fin("water table.txt");
		
		fin>>j;
		
		a = new double[j];
		c = new double[j];
		
		for(i=0;i<j;i++)
		{
			fin>>a[i];
			fin>>c[i];
			if(a[i]==Tw)
			{
				hw = c[i];
				break;
			}
		}
		
		for(i=0;i<j-1;i++)
		{
			
			if(a[i]<Tw&&Tw<a[i+1])
			{
				hw = ((Tw-a[i])/(a[i+1]-a[i]))*(c[i+1]-c[i]) + c[i];
				break;
			}
		}
		fin.close();
		delete [] a;
		delete [] c;
	}

	virtual void CalQ() = 0;
	virtual void Calmdw() = 0;
};

//--1st Grand child class--- Is a child of 2nd Child------------------------------------//
class Cool_Dehum : public HumandDeHum_HandC
{
public:

	Cool_Dehum (int n)
	{
		ACS::newing(n);
	}

	~Cool_Dehum()
	{
		ACS::deleting();
	}
	
	void set_Fi2()
	{
		Fi[1] = 1;
	}

	virtual void Calmdw()
	{
		mdw = abs(mda[0]*(w[1] - w[0]));
	}

	virtual void CalQ()
	{
		Q = mda[0]*(h[1] - h[0]) + mdw * hw;
	}
	
	virtual void Print_info()
	{
		cout<<endl<<"Temperature 1 = "<<T[0]<<" (C)"<<endl<<"Temperature 2 = "<<T[1]<<" (C)"<<endl<<"Water temperature = "<<Tw<<" (C)"<<endl<<endl;
		cout<<"Relative humidity 1 = "<<Fi[0]<<endl<<"Relative humidity 2 = "<<Fi[1]<<endl<<endl;
		cout<<"Mass flow rate of dry air 1 = "<<mda[0]<<" (Kg/s)"<<endl<<"Mass flow rate of dry air 2 = "<<mda[1]<<" (Kg/s)"<<endl<<
			"Mass flow rate of water  = "<<mdw<<endl<<endl;
		cout<<"......................................................................."<<endl;
		cout<<"Saturated pressure 1 = "<<Pg[0]<<" (KPa)"<<endl<<"Saturated pressure 2 = "<<Pg[1]<<" (KPa)"<<endl<<endl;
		cout<<"Absolute humidity 1 = "<<w[0]<<" (Kg/Kg dry air)"<<endl<<"Absolute humidity 2 = "<<w[1]<<" (Kg/Kg dry air)"<<endl<<endl;
		cout<<"Enthalpy 1 = "<<h[0]<<" (KJ/Kg)"<<endl<<"Enthalpy 2 = "<<h[1]<<" (KJ/Kg)"<<endl<<"Enthalpy of water = "<<hw<<" (Kj\Kg)"<<endl<<endl;
		cout<<"Rate of heat transfer = "<<Q<<" (KJ/Kg)"<<endl<<endl;
	}
};

//--2nd Grand child class--- Is a child of 2nd Child------------------------------------//
class Heat_Hum : public HumandDeHum_HandC
{
public:

	Heat_Hum (int n)
	{
		ACS::newing(n);
	}

	~ Heat_Hum()
	{
		ACS::deleting();
	}

	void set_mda3 ()
	{
		mda[2] = mda [0];
	}

	virtual void Calmdw()
	{
		mdw = mda[2]*(w[2] - w[1]);
	}

	virtual void CalQ()
	{
		Q = mda[2]*(h[2] - h[0]) - mdw*hw;
	}
	
	virtual void Print_info()
	{
		cout<<endl<<"Temperature 1 = "<<T[0]<<" (C)"<<endl<<"Temperature 2 = "<<T[1]<<" (C)"<<endl<<"Temperature 3 = "<<T[2]<<" (C)"<<endl<<
			"Water temperature = "<<Tw<<" (C)"<<endl<<endl;
		cout<<"Relative humidity 1 = "<<Fi[0]<<endl<<"Relative humidity 2 = "<<Fi[1]<<endl<<"Relative humidity 3 = "<<Fi[2]<<endl<<endl;
		cout<<"Mass flow rate of dry air 1 = "<<mda[0]<<" (Kg/s)"<<endl<<"Mass flow rate of dry air 2 = "<<mda[1]<<" (Kg/s)"<<endl<<
			"Mass flow rate of dry air 3 = "<<mda[2]<<" (Kg/s)"<<endl<<"Mass flow rate of water = "<<mdw<<endl<<endl;
		cout<<"......................................................................."<<endl;
		cout<<"Saturated pressure 1 = "<<Pg[0]<<" (KPa)"<<endl<<"Saturated pressure 2 = "<<Pg[1]<<" (KPa)"<<endl<<
			"Saterated pressure 3 = "<<Pg[2]<<" (KPa)"<<endl<<endl;
		cout<<"Absolute humidity 1 = "<<w[0]<<" (Kg/Kg dry air)"<<endl<<"Absolute humidity 2 = "<<w[1]<<" (Kg/Kg dry air)"<<endl<<
			"Absolute humidity 3 = "<<w[2]<<" (Kg/Kg dry air)"<<endl<<endl;
		cout<<"Enthalpy 1 = "<<h[0]<<" (KJ/Kg)"<<endl<<"Enthalpy 2 = "<<h[1]<<" (KJ/Kg)"<<endl<<"Enthalpy 3 = "<<h[2]<<" (KJ/Kg)"<<endl<<"Enthalpy of water = "<<hw<<
			" (Kj\Kg)"<<endl<<endl;
		cout<<"Rate of heat transfer = "<<Q<<" (KJ/s)"<<endl<<endl;
	}
};

//------------------------------End of classes------------------------------------------//

//--------------------------------------------------------------------------------------//
int question()
{
	int n;
	cout<<"Do you want to continue?(Yes = 1 , No = 0)";
	cin>>n;
	cout<<"--------------------------------------------------------------------------------"<<endl;
	cout<<"---------------------------------------WARNING----------------------------------"<<endl;
	cout<<"--------All rights reserved by Masoud Vahidi and Amirhossein Borji (-;-----------"<<endl;
	if(n!=1)
	{
		n=0;
	}
	return n;
}
//---------------------------------------------------------------------------------------//

int main()
{
	ACS * ac;
	int m, ggh = 1;
	
	while(ggh)
	{	
		cout<<"Choose Air Conditioning System that you want : "<<endl;
		cout<<"Press 1 for simple heating or cooling"<<endl<<
			  "Press 2 for cooling and dehumidifing"<<endl<<
			  "Press 3 for heating and humidifing"<<endl;
		cin>>m;
		cout<<endl<<endl;
		
		switch(m)
		{
		case(1):
			{
				ac = new simple_HandC(2);
				break;
			}
		case(2):
			{
				ac = new Cool_Dehum(2);
				break;
			}
		case(3):
			{
				ac = new Heat_Hum(3);
				break;
			}
		default:
			{
				cout<<" Error )-: "<<endl;
				ggh = question();
			}
		}
    ac->Solve();


	}
	
	int z;
	cin>>z;
	return 7;
}
 